<?php

namespace common\logic\Distribution\ProductDistribution;

use common\logic\BaseBuilder;
use common\logic\BuilderInterface;
use common\logic\Distribution\Distribution\Distribution;
use common\logic\Distribution\Distribution\DistributionSolver;
use common\logic\Product\Product\Product;
use common\logic\Product\Product\ProductSolver;

class ProductDistributionBuilder extends BaseBuilder implements BuilderInterface
{
    protected ProductDistributionRepository $productDistributionRepository;
    protected DistributionSolver $distributionSolver;
    protected ProductSolver $productSolver;

    public function __construct()
    {
        $this->productDistributionRepository = $this->loadService(ProductDistributionRepository::class);
        $this->distributionSolver = $this->loadService(DistributionSolver::class);
        $this->productSolver = $this->loadService(ProductSolver::class);
    }

    public function instanciateProductDistribution(Distribution $distribution, Product $product): ProductDistribution
    {
        $productDistribution = new ProductDistribution();
        $productDistribution->populateFieldObject('id_distribution', 'distribution', $distribution);
        $productDistribution->populateFieldObject('id_product', 'product', $product);

        return $productDistribution;
    }

    public function createProductDistribution(Distribution $distribution, Product $product): ProductDistribution
    {
        $productDistribution = $this->instanciateProductDistribution($distribution, $product);

        $this
            ->initActive($productDistribution)
            ->initQuantityMax($productDistribution);

        $this->saveCreate($productDistribution);

        return $productDistribution;
    }

    public function createProductDistributionIfNotExist(Distribution $distribution, Product $product): ProductDistribution
    {
        return $this->productDistributionRepository->findOneProductDistribution($distribution, $product)
            ?? $this->createProductDistribution($distribution, $product);
    }

    public function initActive(ProductDistribution $productDistribution): ProductDistribution
    {
        $productDistribution->active = $this->productSolver->isProductActiveByDay(
            $productDistribution->product,
            $this->distributionSolver->getDistributionDayAsString($productDistribution->distribution)
        );

        return $productDistribution;
    }

    public function initQuantityMax(ProductDistribution $productDistribution): ProductDistribution
    {
        $productDistribution->quantity_max = $this->productSolver->getProductQuantityMaxByDay(
            $productDistribution->product,
            $this->distributionSolver->getDistributionDayAsString($productDistribution->distribution)
        );

        return $productDistribution;
    }

}