<?php

namespace common\logic\Subscription\Subscription;

use common\helpers\GlobalParam;
use common\logic\BaseService;
use common\logic\Distribution\Distribution\Distribution;
use common\logic\Producer\Producer\Producer;
use common\logic\RepositoryInterface;
use common\logic\User\User\User;

class SubscriptionRepository extends BaseService implements RepositoryInterface
{
    protected SubscriptionSolver $subscriptionSolver;

    public function __construct()
    {
        $this->subscriptionSolver = $this->loadService(SubscriptionSolver::class);
    }

    public function getDefaultOptionsSearch(): array
    {
        return [
            'with' => ['producer'],
            'join_with' => ['user', 'productSubscription', 'productSubscription.product', 'pointSale'],
            'orderby' => 'user.name ASC',
            'attribute_id_producer' => 'subscription.id_producer'
        ];
    }

    public function findOneSubscriptionById(int $id): ?Subscription
    {
        return Subscription::searchOne(['id' => $id]);
    }

    public function findSubscriptions()
    {
        return Subscription::searchAll();
    }

    /**
     * Retourne les abonnements pour une date donnée.
     */
    // searchByDate, getByDate
    public function findSubscriptionsByDate(string $date)
    {
        $date = date('Y-m-d', strtotime($date));
        $subscriptionArray = $this->findSubscriptions();
        $subscriptionReturnArray = [];

        foreach ($subscriptionArray as $subscription) {
            if ($date >= $subscription->date_begin &&
                (!$subscription->date_end || $date <= $subscription->date_end)
                && $this->subscriptionSolver->isSubscriptionMatchWith($subscription, $date)) {
                $subscriptionReturnArray[] = $subscription;
            }
        }

        return $subscriptionReturnArray;
    }

    public function countSubscriptionsByUser(User $user, Producer $producer)
    {
        return Subscription::find()
            ->where([
                'subscription.id_user' => $user->id,
                'subscription.id_producer' => $producer->id,
            ])->count();
    }
}